import json
import yaml
from collections import OrderedDict

class OutputWriterPerConsensus(object):

    def __init__(self):
        self._hasCodingPart = False
        self._hasStructPart = False
        self._hasOtherPart = False
    
    def writeOutput(self, outF, dResult):
        lMandatoryKeys = ['consensusName','consensusLength','consensusStrand','chimeric','consensusClass','consensusOrder','completeness']
        lKeys2Write = []
        lCodingAndStructAndOther2Write = []
        
        for key in lMandatoryKeys:
            lKeys2Write.append(dResult[key])

        confidenceIndex = dResult['confidenceIndex']
        confidenceIndex = 'CI=' + str(confidenceIndex)
        lKeys2Write.append(confidenceIndex)
        if dResult.has_key('dictOfEvidence'):
            dEvidence = dResult['dictOfEvidence']
            coding = self.writeCodingFeaturesLine(dEvidence)
            if coding != "":
                lCodingAndStructAndOther2Write.append(coding)
            
            struct = self.writeStructFeaturesLine(dEvidence)
            if struct != "":
                lCodingAndStructAndOther2Write.append(struct)
            
            other = self.writeOtherFeaturesLine(dEvidence)
            if other != "":
                lCodingAndStructAndOther2Write.append(other)
        outF.write('\t'.join(lKeys2Write) + "; " + "; ".join(lCodingAndStructAndOther2Write) + '\n')
        return self.generateJsonClassifDict(dResult)

        
    def generateJsonClassifDict(self, dResult):
        def set_default(obj):
            if isinstance(obj, set):
                return list(obj)
            raise TypeError
        
        from json import JSONEncoder

        def _default(self, obj):
            return getattr(obj.__class__, "to_json", _default.default)(obj)

        _default.default = JSONEncoder().default # save unmodified default
        JSONEncoder.default = _default # replacement
        
        tmpDict = dResult.copy()
        
        cleanedDict = OrderedDict()
        cleanedDict["name"] = tmpDict["consensusName"]
        cleanedDict["length"] = int(tmpDict["consensusLength"])
        cleanedDict["strand"] = tmpDict["consensusStrand"]
        cleanedDict["chimeric"] = False if( tmpDict["chimeric"] == "ok" )  else True
        
        cleanedDict["class"] = tmpDict["consensusClass"]
        cleanedDict["order"] = tmpDict["consensusOrder"]
        cleanedDict["complete"] = True if( tmpDict["completeness"] == "complete") else (False if (tmpDict["completeness"] == "incomplete") else None)
        cleanedDict["CI"] = tmpDict["confidenceIndex"]
        
        if tmpDict.has_key('dictOfEvidence'):
            dEvidence = tmpDict['dictOfEvidence']

            if self._hasCodingPart:
                cleanedDict["coding"] = OrderedDict()
                self.formatCodingFeaturesAsDict(dEvidence, cleanedDict["coding"])
            if self._hasStructPart:
                cleanedDict["struct"] = OrderedDict()
                self.formatStructFeaturesAsDict(dEvidence, cleanedDict["struct"])
            if self._hasOtherPart:
                cleanedDict["other"] = OrderedDict()
                self.formatOtherFeaturesAsDict(dEvidence, cleanedDict["other"])
#        Delete key if value is an empty ordered dictionnary
        for key, value in cleanedDict.iteritems():
            if value == OrderedDict() :
                del cleanedDict[key]

        return cleanedDict
        

    def writeOtherFeaturesLine(self, dEvidence): 
        other = ""
        if dEvidence.has_key('other'):
                lResults = []
                dOtherResults = dEvidence['other']
                lResultsWithCoding = self.formatCodingFeatures(dOtherResults, lResults)
                lResultsFilled = self.formatStructFeatures(dOtherResults, lResultsWithCoding)

                if len(lResultsFilled) != 0:
                    subOther = "; ".join(lResultsFilled)
                    other = 'other=(%s)' % subOther
                    self._hasOtherPart = True
        return other

    def writeCodingFeaturesLine(self, dEvidence):
        lResults = []
        lResultsFilled = self.formatCodingFeatures(dEvidence, lResults)
        coding = ""
        if len(lResultsFilled) != 0:
            subCoding = "; ".join(lResultsFilled)
            coding = 'coding=(%s)' % subCoding
        return coding

    def formatCodingFeatures(self, dEvidence, lResults):
        if dEvidence.has_key('Repbase_tbx') and dEvidence['Repbase_tbx'] != []:
            lResults.append("TE_BLRtx: %s" % ", ".join(map(str, dEvidence['Repbase_tbx'])))
        
        if dEvidence.has_key('Repbase_bx') and dEvidence['Repbase_bx'] != []:
            lResults.append("TE_BLRx: %s" % ", ".join(map(str, dEvidence['Repbase_bx'])))
            
        if (dEvidence.has_key('te_hmmer')) and (dEvidence['te_hmmer'] != None):
            lResults.append('profiles: %s' % self.formatProfilesResults(dEvidence['te_hmmer']))
            
        if dEvidence.has_key('Other_profiles'):
            lResults.append('Other_profiles: %s' % self.formatProfilesResults(dEvidence['Other_profiles']))
        
        if dEvidence.has_key("rDNA") and (dEvidence["rDNA"] != None):
            lResults.append("rDNA_BLRn: %s" % dEvidence["rDNA"])
        
        if dEvidence.has_key("HG") and (dEvidence["HG"] != None):
            lResults.append("HG_BLRn: %s" % dEvidence["HG"])
        
        if len(lResults) != 0:
            self._hasCodingPart = True
        return lResults

    def formatProfilesResults(self, dProfilesResults):
        if len(dProfilesResults.keys()) == 0:
            return ""
        lResults = []
        for key in dProfilesResults.keys():
            iPDM = dProfilesResults[key]
            cov = "%.2f%%(%.2f%%)" % (iPDM.getCoverage(),iPDM.getCoverageOnSubject() )
            profilesResult = '%s: %s' % (key, cov)
            lResults.append(profilesResult)
        return ", ".join(lResults)

    
    def writeStructFeaturesLine(self, dEvidence):
        lResults = []
        lResultsFilled = self.formatStructFeatures(dEvidence, lResults)
        struct = ""

        if len(lResultsFilled) != 0:
            subStruct = "; ".join(lResultsFilled)
            struct = 'struct=(%s)' % subStruct
        return struct

    def formatStructFeatures(self, dEvidence, lResults):
        if dEvidence.has_key('length') and (dEvidence['length']!= None):
            lResults.append('TElength: %s' % dEvidence['length'])

        if dEvidence.has_key('TR') and (dEvidence['TR'] != None):
            lResults.append('TermRepeats: %s' % ", ".join(map(str, dEvidence['TR'])))    
            
        if dEvidence.has_key('ORF') and (dEvidence['ORF'] != None):
            lResults.append('ORF: %s' % ", ".join(dEvidence['ORF']))        

        if dEvidence.has_key('SSR') and (dEvidence['SSR'] != None):
            lResults.append('SSR: %s' % ", ".join(dEvidence['SSR']))
        
        if dEvidence.has_key('SSRCoverage') and (dEvidence['SSRCoverage'] != None) :
            lResults.append('SSRCoverage=%s' % dEvidence['SSRCoverage'])
            
        if dEvidence.has_key('polyAtail'):
            lResults.append('polyAtail')   
             
        if dEvidence.has_key('helitronExtremities') and (dEvidence['helitronExtremities'] != None):
            lResults.append('helitronExtremities: %s' % ", ".join(map(str, dEvidence['helitronExtremities'])))
        if len(lResults) != 0:
            self._hasStructPart = True
        
        return lResults


    def formatProfilesResultsAsDict(self, dProfilesResults):
        if len(dProfilesResults.keys()) == 0:
            return {}
        
        dResults = {}
        for key in dProfilesResults.keys():
            iPDM = dProfilesResults[key]
            profilesResult = OrderedDict()
            profilesResult["cov"] = iPDM.getCoverage()
            profilesResult["covOnSubject"] = iPDM.getCoverageOnSubject()
            #profilesResult = '%s: %s' % (key, cov)
            dResults[key] = profilesResult
        return dResults

    def formatCodingFeaturesAsDict(self, dEvidence, dCoding):
        if dEvidence.has_key('Repbase_tbx') and dEvidence['Repbase_tbx'] != []:
            dCoding["TE_BLRtx"] = OrderedDict()
            
            for iBM in  dEvidence['Repbase_tbx']:
                profilesResult = OrderedDict()
                profilesResult["cov"] = iBM.getCoverage()
                dCoding["TE_BLRtx"][iBM.getName()] = profilesResult
    
        if dEvidence.has_key('Repbase_bx') and dEvidence['Repbase_bx'] != []:
            dCoding["TE_BLRx"] = OrderedDict()
            for iBM in  dEvidence['Repbase_bx']:
                profilesResult = OrderedDict()
                profilesResult["cov"] = iBM.getCoverage()
                dCoding["TE_BLRx"][iBM.getName()] = profilesResult
            
        if (dEvidence.has_key('te_hmmer')) and (dEvidence['te_hmmer'] != None):
            dCoding["profiles"] = self.formatProfilesResultsAsDict(dEvidence['te_hmmer'])
          
        if dEvidence.has_key('Other_profiles'):
            dCoding["Other_profiles"] = self.formatProfilesResultsAsDict(dEvidence['Other_profiles'])

        if dEvidence.has_key("rDNA") and (dEvidence["rDNA"] != None):
            dCoding["rDNA_BLRn"] = OrderedDict()
            iBM =  dEvidence['rDNA']
            dCoding["rDNA_BLRn"]["name"] = iBM.getName()
            dCoding["rDNA_BLRn"]["cov"] = iBM.getCoverage()
            
        if dEvidence.has_key("HG") and (dEvidence["HG"] != None):
            dCoding["HG_BLRn"] = OrderedDict()
            iBM =  dEvidence['HG']
            dCoding["HG_BLRn"]["cov"] = iBM.getCoverage()
            dCoding["HG_BLRn"]["name"]= iBM.getName()

  
    def formatStructFeaturesAsDict(self, dEvidence, dStruct):
        if dEvidence.has_key('length') and (dEvidence['length']!= None):
            dStruct["TElength"] = dEvidence['length']

        if dEvidence.has_key('TR') and (dEvidence['TR'] != None):
            dStruct["TermRepeats"] = OrderedDict()
            for iTR in  dEvidence['TR']:
                trResult = OrderedDict()
                trResult["length"] = iTR.getLength()
                dStruct["TermRepeats"][iTR.getType()] = trResult
            
        if dEvidence.has_key('ORF') and (dEvidence['ORF'] != None):
            dStruct["ORF"] = OrderedDict()
            for iORF in dEvidence['ORF']:
                dStruct["ORF"]= dEvidence['ORF']
            
        if dEvidence.has_key('SSR') and (dEvidence['SSR'] != None):
            dStruct["SSR"] = OrderedDict()
            for iORF in dEvidence['SSR']:
                dStruct["SSR"]= dEvidence['SSR']
                
        if dEvidence.has_key('SSRCoverage') and (dEvidence['SSRCoverage'] != None) :
            dStruct["SSRCoverage"] = dEvidence['SSRCoverage']
      
        if dEvidence.has_key('polyAtail'):
            dStruct["polyAtail"] = True if( dEvidence['polyAtail'] == "yes" )  else False
             
        if dEvidence.has_key('helitronExtremities') and (dEvidence['helitronExtremities'] != None):
            dStruct["helitronExtremities"] = OrderedDict()
            
            for iHE in  dEvidence['helitronExtremities']:
                helitronExtResult = OrderedDict()
                helitronExtResult["start"] = iHE.getStart()
                helitronExtResult["end"] = iHE.getEnd()
                helitronExtResult["eValue"] = iHE.getEValue()
                dStruct["helitronExtremities"][iHE.getName()] = helitronExtResult
            

    def formatOtherFeaturesAsDict(self, dEvidence, dOther):
        dOtherEvidences = dEvidence['other']
        self.formatCodingFeaturesAsDict(dOtherEvidences, dOther)
        self.formatStructFeaturesAsDict(dOtherEvidences, dOther)
        
