#!/usr/bin/env python


import subprocess, tempfile, sys, os, glob, shutil
from optparse import OptionParser


class wrapKissplice(object):

	def __init__(self):
		self._options = None
		
	def getSystemCommand(self, prg, lArgs):
		systemCmd = prg 
		for arg in lArgs:
			systemCmd += " " + arg
		return systemCmd

	def setOptions(self, options):
		self._options = options

	def stop_err(self, msg ):
		sys.stderr.write( "%s\n" % msg )
		sys.exit()

	def setAttributesFromCmdLine(self):
		parser = OptionParser(description = "Kissplice for Galaxy", version = "v2.0")
		parser.add_option( "-D", "--dir", dest = "dir", help = "dir for Galaxy")
		parser.add_option( "-r", "--reads", dest = "reads", action = 'append', help = "reads file name")
		parser.add_option( "-i", "--id", dest = "id", help = "Galaxy id")
		parser.add_option( "-k", "--kmer", dest = "kmer", default=25, help = "k-mer size (default 25)"),
		parser.add_option( "-l", "--llmax", dest = "llmax", default=0, help = "maximal length of the shorter path (default: 2k-1)"),
		parser.add_option('-m', action = "store", dest = "LL_MIN", default = 0, help = "minimum length of the shorter path (default 2k-8)")
		parser.add_option('-M', action = "store", dest = "UL_MAX", default = 1000, help = "maximum length of the longest path (default 1000), skipped exons longer than UL_MAX are not reported")
		parser.add_option('-g', action="store", dest="graph_prefix", default="", help="path and prefix to pre-built de Bruijn graph (suffixed by .edges/.nodes)\n if jointly used with -r, graph used to find bubbles and reads used for quantification")
		parser.add_option('-o', action="store", dest="out_dir", default="results", help="path to store the results (default = ./results)")
		parser.add_option('-d', action="store", dest="path_to_tmp", default=None, help="specific directory (absolute path) where to build temporary files (default temporary directory otherwise)")
		parser.add_option('-t', action="store", dest="nbprocs", type=int, default=1, help="number of cores (must be <= number of physical cores)")
		parser.add_option('-s', action="store_true", dest="nooutput_snps", help="Will not ouput SNPs (save time)")
		parser.add_option('-v', action="store_true", dest="verbose", help="Verbose mode")
		parser.add_option('-u', action="store_true", dest="keep_ubccs",help="keep the nodes/edges file for unfinished bccs")
		parser.add_option('-c',  action = "store", dest = "min_cov", default = 2, help="an integer, k-mers present strictly less than this number of times in the dataset will be discarded (default 2)")
		parser.add_option('-C',  action = "store", dest = "min_relative_cov", default = 0.02, help="a percentage from [0,1), edges with relative coverage below this number are removed (default 0.02)")
		parser.add_option('-z', action="store", dest="genome_size", type=int, default=1000000000, help="estimated genome/transcriptome size (default = 1000000000)")
		parser.add_option('-e',  action = "store", dest = "min_edit_dist", default = 3, help="edit distance threshold, if the two sequences (paths) of a bubble have edit distance smaller than this threshold, the bubble is classified as an inexact repeat (default 3)")
		parser.add_option('-y',  action = "store", dest = "max_cycles", default = 10000, help="maximal number of bubbles enumeration in each bcc. If exceeded, no bubble is output for the bcc (default 10000)")
		parser.add_option('--timeout', action='store', dest="timeout", default=900, help="max amount of time (in seconds) spent for enumerating bubble"),
		parser.add_option( "-L", "--log", dest = "log", help = "log")
		options, args = parser.parse_args()
		fh = open(options.log, "w")
		fh.write("You need to refresh Galaxy History to see the output files")
		fh.close()
		self._setAttributesFromOptions(options)

	def _setAttributesFromOptions(self, options):
		self.setOptions(options)


	def run(self):
		prg = "kissplice"
		args = []
		for i, opt in enumerate(self._options.reads):
			base = os.path.basename(opt)
			fastq = "%s.fq" % base
			os.symlink(opt, fastq)
			args.append("-r %s" % fastq)
		args.append("-k %s" % self._options.kmer)
		args.append("-l %s" % self._options.llmax)
		args.append("-m %s" % self._options.LL_MIN)
		args.append("-M %s" % self._options.UL_MAX)
		if self._options.graph_prefix != "":
			args.append("-g %s" % self._options.graph_prefix)
		args.append("-o %s" % self._options.out_dir)
		if self._options.path_to_tmp:
			args.append("-d %s" % self._options.path_to_tmp)
		args.append("-t %s" % self._options.nbprocs)
		if self._options.nooutput_snps:
			args.append("-s %s" % self._options.nooutput_snps)
		if self._options.verbose:
			args.append("-v %s" % self._options.verbose)
		if self._options.keep_ubccs:
			args.append("-u %s" % self._options.keep_ubccs)
		args.append("-c %s" % self._options.min_cov)
		args.append("-C %s" % self._options.min_relative_cov)
		args.append("-z %s" % self._options.genome_size)
		args.append("-e %s" % self._options.min_edit_dist)
		args.append("-y %s" % self._options.max_cycles)
		args.append("--timeout %s" % self._options.timeout)
		cmd = self.getSystemCommand(prg, args)
		
		try:
			tmp_err = tempfile.NamedTemporaryFile().name
			tmp_stderr = open( tmp_err, 'wb' )
			proc = subprocess.Popen( args=cmd, shell=True, cwd=".", stderr=tmp_stderr )
			returncode = proc.wait()
			tmp_stderr.close()
			# get stderr, allowing for case where it's very large
			tmp_stderr = open( tmp_err, 'rb' )
			stderr = ''
			buffsize = 1048576
			try:
				while True:
					stderr += tmp_stderr.read( buffsize )
					if not stderr or len( stderr ) % buffsize != 0:
						break
			except OverflowError:
				pass
			tmp_stderr.close()
			if returncode != 0:
				raise Exception, stderr
		except Exception, e:
			self.stop_err( 'Error in kissplice:\n' + str( e ) ) 
			
		os.chdir("results")
		for results in glob.glob("*.fa"):
			baseName = os.path.basename(results)	
			split = os.path.splitext(baseName)
			alias =split[0].replace("_", "")
			shutil.move("%s" % (results) , "%s/primary_%s_%s_visible_fa" % (self._options.dir, self._options.id, alias))
		os.chdir("../")

if __name__ == "__main__":
	untar = wrapKissplice()
	untar.setAttributesFromCmdLine()
	untar.run()	
