#!/usr/bin/env python
from optparse import OptionParser
import os
import sys
import re
from SMART.Java.Python.misc.RPlotter import RPlotter
from commons.core.utils.FileUtils import FileUtils
from commons.core.checker.RepetException import RepetException


class PlotBenchMarkGFFFiles(object):

    def __init__(self, inputFileName = "", ouputFileName = "", attribute = "identity", maxLength = "", type = "barplot", imageWidth=2000, imageHeight=1000,xLabel="",yLabel="",rawData=None, title=""):
        self._options = None
        self._inputFileName = inputFileName
        self._outFileName = ouputFileName
        self._attribute = attribute
        self._type = type
        self._maxLength = maxLength
        self._imageWidth = imageWidth
        self._imageHeight = imageHeight
        
        self._xLabel = xLabel
        self._yLabel = yLabel
        self._rawData = rawData
        self._title = title

    def help(self):
        print 
        print "usage: %s [ options ]" % ( sys.argv[0] )
        print "options:"
        print "     -h: this help"
        print "     -i: input file [format: gff3]"
        print "     -o: output file [format: png]"
        print "     -a: attribute to plot"
        print "     -t: graph type [histogram: default | ]"
        print
        print "Examples:"
        print
        print "1:\n\tpython PlotSwFiles.py -i inputFile -o outputFile "
        print
        print

#TODO: Extract legend X and Y axis in options
    def setAttributesFromCmdLine(self):
        description = "PlotSwFiles plots lines from a list of files for the same data."
        
        parser = OptionParser(description = description)
        parser.add_option("-i", "--input", dest="inputFileName", default = None,  action="store", type="string", help="input file [compulsory] [format: gff3]")
        parser.add_option("-o", "--pictureName", dest="outputFileName", default = None, action="store", help="output file [compulsory] [format: png]")
        parser.add_option("-a", "--attribute", dest="attribute", default = None, action="store", help="attribute to plot")
        parser.add_option("-t", "--type", dest="type", default = "barplot", action="store", help="plot type")
        parser.add_option("-m", "--maxLength", dest="maxLength", default = "", action="store", help="Max X axis length")
        parser.add_option("-n", "--title", dest="title", default = "", action="store", help="graph title")
        (self._options, args) = parser.parse_args()
        self._inputFileName = self._options.inputFileName
        self._outFileName = self._options.outputFileName
        self._attribute = self._options.attribute
        self._type = self._options.type
        self._maxLength = self._options.maxLength
        self._title = self._options.title

    def checkOptions(self):
#        if self._inputFileName != None:
#            if not FileUtils.isRessourceExists(self._inputFileName):
#                msg = "ERROR: file %s doesn't exist!" % self._inputFileName
#                sys.stderr.write( "%s\n" % msg )
#                sys.stderr.flush()
#                sys.exit(1)
#        else:
#            msg = "ERROR: Option input file missing!"
#            sys.stderr.write( "%s\n" % msg )
#            sys.stderr.flush()
#            sys.exit(1)
            
        if self._outFileName == None:
            msg = "ERROR: Option output file missing!"
            sys.stderr.write( "%s\n" % msg )
            sys.stderr.flush()
            sys.exit(1)
        
    def setFilesListFileName(self, fileName):
        self._inputFileName = fileName
        
    def setOutFileName(self, fileName):
        self._outFileName = fileName

    def extract_repartition(self,fileHandler):
        data = {}
        line = fileHandler.readline() 
        ranges = {}
        while line :
            try:
                start, end, valueToPlot = self.getStartEndAndValueOfAttribute(line)
                irange = int(float(valueToPlot))
                if ranges.get(irange,None) is None:
                    #ranges[range] = []
                    ranges[irange] = 0 
                ranges[irange] += 1
                
                Xplot = (start + end)/2
                Yplot = valueToPlot
                if Yplot != "NA":
                    Yplot = float(Yplot)
                data[Xplot] = Yplot
                line = fileHandler.readline()
            except RepetException, e:
                print e.getMessage()
                
        for n in range(0,100):
            if not n in ranges.keys():
                ranges[n]=0
     
        return ranges
    
    
    def File2Dict (self, fileHandler):
        dX2Y = {}
        line = fileHandler.readline() 
        while line :
            try:
                start, end, valueToPlot = self.getStartEndAndValueOfAttribute(line)
                Xplot = (start + end)/2
                Yplot = valueToPlot
                if Yplot != "NA":
                    Yplot = float(Yplot)
                dX2Y[Xplot] = Yplot
                line = fileHandler.readline()
            except RepetException, e:
                print e.getMessage()
        return dX2Y
    
    def get_repartition(self,step=1):
        """ get total number of something in the given range"""
        pass
    
    def getStartEndAndValueOfAttribute(self, line):
        valueToPlot = None
        line = line.rstrip()
        start = int(line.split('\t')[3])
        end = int(line.split('\t')[4])
        lAttributes = line.split('\t')[8].split(";")
        for attribute in lAttributes:
            if re.search(self._attribute, attribute):
                valueToPlot = attribute.replace('%s=' % self._attribute, '')
        if valueToPlot == None:
            raise Exception("Problem value to plot %s is not numerical value" % self._attribute)
        return start, end, valueToPlot
    
    def getStartEndAndValueOfAttribute2(self, line, attributeToSearch):
        valueToPlot = None
        line = line.rstrip()
        start = int(line.split('\t')[3])
        end = int(line.split('\t')[4])
        lAttributes = line.split('\t')[8].split(";")
        for attribute in lAttributes:
            if re.search(attributeToSearch, attribute):
                valueToPlot = attribute.replace('%s=' % attributeToSearch, '')
        if valueToPlot == None:
            raise Exception("Problem value to plot %s is not numerical value" % attributeToSearch)
        return start, end, valueToPlot

    def run(self):
        self.checkOptions()
        self._iRPlotter = RPlotter(self._outFileName)
        #TODO: move the legend to have the whole source name
        self._iRPlotter.setTitle(self._title)
        self._iRPlotter.setLegend(True)
        #self._iRPlotter.setMinimumY(1)
        #self._iRPlotter.setMaximumY(100)
        
        #self._iRPlotter.setMinimumX(0)
        #self._iRPlotter.setMaximumX(100)
        
        #usefull
        #self._iRPlotter.yMax=100
        self._iRPlotter.yMin = 0
        self._iRPlotter.setBuckets(1)
        
        #file = open(self._inputFileName, "r")
        #dX2Y = self.extract_repartition(file)
        
        dX2Y = self._rawData
        if self._type == "point":
            self._iRPlotter.setPoints(True)
        elif self._type == "heatPoints":
            self._iRPlotter.setHeatPoints(True)
        else:
            self._iRPlotter.setBarplot(True)
        self._iRPlotter.setXLabel(self._xLabel)
        self._iRPlotter.setYLabel(self._yLabel)
        self._iRPlotter.addLine(dX2Y)
        if self._maxLength != "":
            self._iRPlotter.setMaximumX(self._maxLength)
        self._iRPlotter.setTitle(os.path.basename(self._inputFileName).split(".")[0])
        self._iRPlotter.setImageSize(self._imageWidth,self._imageHeight)
        self._iRPlotter.addTopMargin(0.05)
        self._iRPlotter.plot()
        #file.close()
        
if __name__ == "__main__":
    iPlotSwFiles = PlotBenchMarkGFFFiles()
    iPlotSwFiles.setAttributesFromCmdLine()
    iPlotSwFiles.run()