# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

"""
Utility to handle a databank of HMM profiles.
"""

import os
import sys
from commons.core.utils.FileUtils import FileUtils
from commons.pyRepetUnit.profilesDB.Profiles import Profiles
from commons.pyRepetUnit.profilesDB.ProfilesDatabank import ProfilesDatabank

class ProfilesDatabankUtils:
    """
    Utility to handle a databank of HMM profiles.
    """
    @staticmethod
    def read(inFileName, verbose = 0):
        """
        Read a file in Pfam format and return a L[ProfilesDatabank<commons.core.ProfilesDatabank>} instance.
        @param inFileName: name of the input file
        @type inFileName: string
        @param verbose: verbosity level
        @type verbose: integer
        """ 
        if verbose > 0: print "reading file '%s'..." % inFileName; sys.stdout.flush()
        
        if FileUtils.isEmpty(inFileName):
            return (None)
        profilesInstance = Profiles() 
        profilesDBInstance = ProfilesDatabank()
        f = open(inFileName, "r")
        while profilesInstance.read(f):
            profilesDBInstance.append(profilesInstance)
            profilesInstance = Profiles()
        f.close()
        if verbose > 0: print "file '%s' is loaded" % inFileName; sys.stdout.flush()
        return (profilesDBInstance)
    
    ## Split a single HMM files into multiple files
    #
    @staticmethod
    def splitFile(inputFile, maxNbProfilesPerFile = 1):
        basename = os.path.splitext(os.path.basename(inputFile))[0]
        extension = os.path.splitext(os.path.basename(inputFile))[1]
        outputFileIndex = 1
        generatedFileNames = []
        
        inputF = open(inputFile, 'r')
        
        outputFileName = "%s%d%s" % (basename, outputFileIndex, extension)
        generatedFileNames.append(outputFileName)
        currentOutputFile = open(outputFileName, 'w')
            
        nbProfilesInCurrentFile = 0
        line = inputF.readline()
        while line:
            if '//' in line:
                nbProfilesInCurrentFile += 1
                currentOutputFile.write(line)
                line = inputF.readline()
                
                if nbProfilesInCurrentFile >= maxNbProfilesPerFile:
                    currentOutputFile.close()
                    outputFileIndex += 1
                    outputFileName = "%s%d%s" % (basename, outputFileIndex, extension)
                    generatedFileNames.append(outputFileName)
                    currentOutputFile = open(outputFileName, 'w')
                    nbProfilesInCurrentFile = 0
                    if not line:
                        currentOutputFile.close()
                        lastFile = generatedFileNames.pop()
                        os.remove(lastFile)
            else:
                currentOutputFile.write(line)
                line = inputF.readline()
      
        currentOutputFile.close()
        inputF.close()
        
        return generatedFileNames