from commons.core.sql.TableSetAdaptator import TableSetAdaptator
from commons.core.coord.SetUtils import SetUtils
from PASTEC.utils.AbstractRetrieverAgent import AbstractRetrieverAgent
from PASTEC.DemandAndResult import DemandAndResult
from PASTEC.SSRCollection import SSRCollection
from PASTEC.SSR import SSR
import copy


class SSR_Agent(AbstractRetrieverAgent):
    """
    This agent search in the database type of tandem repeats of TE consensus 
    """

    SUFFIX_TABLE_NAME_LIST = ["_SSR_set"] 
    
    def __init__(self, env):
        AbstractRetrieverAgent.__init__(self, env)
        self._lSSR = None
        self._lSSRExtremities = None
        self._coverage = None
          
    def analyse(self, demand, lSet):
        consensusLength = self.askForConsensusLength(demand.consensusName)
        iSSRCollection = SSRCollection()
        for set in lSet:
            if set.start == 1:
                iSSR = SSR(set.name, "start")
                iSSRCollection.append(iSSR)
            elif set.end == consensusLength:
                iSSR = SSR(set.name, "end")
                iSSRCollection.append(iSSR)
        self._setCoverage(consensusLength, lSet)
        iSSRCollection.setCoverage(self._coverage)
        return DemandAndResult(demand, iSSRCollection) 
    
    def computeCoverage(self, lSet, consensusLength):
        lMergedSets = SetUtils.mergeSetsInList( lSet)
        cumulSSRLength = SetUtils.getCumulLength( lMergedSets )
        coverage = cumulSSRLength / float(consensusLength)
        return coverage
    
    def getListOfFeaturesToAnalyse(self, consensusName):
        tSetA = TableSetAdaptator(self._db, self._tableName)
        lSet = tSetA.getSetListFromSeqName(consensusName)
        return lSet
    
    def _appendlSSRExtremities(self, set, consensusLength):
        if set.start == 1:
            self._lSSRExtremities.append(set.name + "_start")
        if set.end == consensusLength:
            self._lSSRExtremities.append(set.name + "_end")

    def _setCoverage(self, consensusLength, lSet):
        lSetCloned = []
        for set in lSet:
            setCloned = copy.copy(set)
            lSetCloned.append(setCloned)
        self._coverage = str(self.computeCoverage(lSetCloned, consensusLength))

    def _appendlSSR(self, set):
        resultDict = {'name': set.name, 'start': set.start, 'end': set.end}
        self._lSSR.append(resultDict)