from PASTEC.Strand import Strand
from PASTEC.DemandAndResult import DemandAndResult
from PASTEC.Demand import Demand



class Strand_Agent(object):

    ## Constructor
    #
    # @param env Environment instance
    #
    def __init__(self, env):
        self._environment = env
        self._conclusionsMatch = []
        self._listOfKnownAgents = ["TE_BLRx_Agent", "TE_BLRtx_Agent", "TE_Profiles_Agent"]
        self._setAgentDecisionRules()
        
    def reset(self):
        self._conclusionsMatch = []
        
    def getSuffixTableNameList(self):
        return []
        
    def getConclusionsMatch(self):
        return self._conclusionsMatch
    
    def setConclusionsMatch(self, conclusionsMatch):
        self._conclusionsMatch = conclusionsMatch
    
    def getDependentAgentsList(self):
        return self._listOfKnownAgents
    
    def setDependentAgentsList(self, lAgents):
        self._listOfKnownAgents = lAgents
    
    def _setAgentDecisionRules(self):
        """sets the current agen'ts decision rules based on info from configuration file"""
        decisionRulesData = self._environment.getAgentDecisionRules(self.__class__.__name__)
        self.MIN_BLASTER_COVERAGE = decisionRulesData["MIN_BLASTER_COVERAGE"]
        self.MIN_PROFILES_COVERAGE = decisionRulesData["MIN_PROFILES_COVERAGE"]
    
    def setConsensus(self, consensusName):
        self._consensusName = consensusName
        
    def getAnalysesResults(self, demand):
        for agentName in self._listOfKnownAgents:
            iDemand = Demand(agentName, demand.consensusName)
            iAgentResult = self._environment.getResultAccordingToDemand(iDemand)
            method2Call = "self.makeAppropriateConclusionsFor%s( iAgentResult )" % agentName
            exec ( method2Call )
            
    def analyse(self, iDemand):
        strand = "."
        firstProfile = True
        maxBlasterCoverage = 0
        for result in self._conclusionsMatch:
            if self._isBlasterSignificantResult(result, maxBlasterCoverage):
                maxBlasterCoverage = result.getCoverage()
                strand = result.getStrand()
            elif self._isProfileSignificantResult(result, maxBlasterCoverage):
                if firstProfile:
                    strand = result.getStrand()
                    firstProfile = False
                elif strand != result.getStrand():
                    strand = "."
        return DemandAndResult(iDemand, Strand(strand))
        
    ## Three main step : make demands, get results and decide
    #
    def live(self):
        for consensusName in self._environment.getConsensusNameList():
            iDemand = Demand(self.__class__.__name__, consensusName)
            self._consensusName = iDemand.consensusName 
            self.reset()

            self.getAnalysesResults(iDemand)
            iDemandAndResult = self.analyse(iDemand)
            
            self._environment.addResultToBlackBoard(iDemandAndResult)

    def makeAppropriateConclusionsForTE_BLRx_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result != []:
            self._conclusionsMatch.extend(iDemandAndResult.result)
    
    def makeAppropriateConclusionsForTE_BLRtx_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result != []:
            self._conclusionsMatch.extend(iDemandAndResult.result)
    
    def makeAppropriateConclusionsForTE_Profiles_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result != []:
            self._conclusionsMatch.extend(iDemandAndResult.result)
    
    def _isBlasterSignificantResult(self, result, maxBlasterCoverage):
        return str(result.__class__.__name__) == "BlasterMatch" and result.getCoverage() > maxBlasterCoverage and result.getCoverage() > self.MIN_BLASTER_COVERAGE
    
    def _isProfileSignificantResult(self, result, maxBlasterCoverage):
        return maxBlasterCoverage == 0 and result.getCoverage() > self.MIN_PROFILES_COVERAGE