#!/usr/bin/env python

import user, os, sys, getopt, string, re

def setup_env():
    if "REPET_PATH" in os.environ.keys():
        sys.path.append( os.environ["REPET_PATH"] )
setup_env()

from pyRepet.seq.Bioseq import *
from xml.sax import ContentHandler
from xml.sax import ErrorHandler
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces

class FindRep(ContentHandler):
    def __init__(self,fileoutname,filter,count=0):
        self.inWindowContent=0
        self.inSeqNameContent=0
        self.inStartContent=0
        self.inEndContent=0
        self.inPeriodContent=0
        self.inUnitContent=0
        self.inScoreContent=0
        self.count=count
        self.fileout=open(fileoutname,"w")
        self.filter=filter
        
    def startElement(self,name,attrs):
        if name=="window":
            self.inWindowContent=1
        elif name=="sequence-name":
            self.inSeqNameContent=1
            self.seqname=""
        elif name=="repeat":
            self.inRepContent=1
            self.start=""
            self.end=""
            self.period=""
            self.type={}
        elif name=="start":
            self.inStartContent=1
        elif name=="end":
            self.inEndContent=1
        elif name=="period":
            self.inPeriodContent=1
        elif name=="unit":
            self.inUnitContent=1
            self.unit=""
        elif name=="score":
            self.inScoreContent=1
            self.score=""
            
    def characters(self,ch):
        if self.inSeqNameContent:
            self.seqname+=ch
        elif self.inStartContent:
            self.start+=ch
        elif self.inEndContent:
            self.end+=ch
        elif self.inPeriodContent:
            self.period+=ch            
        elif self.inUnitContent:
            self.unit+=ch            
        elif self.inScoreContent:
            self.score+=ch            
            
    def endElement(self,name):
        if name=="window":
            self.inWindowContent=0
        elif name=="sequence-name":
            self.inSeqNameContent=0
        elif name=="repeat":
            self.inRepContent=0
            start=int(self.start)
            end=int(self.end)
            period=int(self.period)
            score=float(self.score)
            if score>self.filter:
                return
            max=0
            self.count+=1
            for k,n in self.type.items():
                if n>max:
                    max=n
                    k_max=k

            m=re.match("^[0-9]+.+\{Cut\}",self.seqname)
            if m!=None:
                seqname=self.seqname[m.start(0):m.end(0)-5].rstrip()
                seqname=re.sub("^[0-9]+ ","",seqname).lstrip()
                tok=self.seqname[m.end(0):].split("..")
                astart=start+int(tok[0])-1
                aend=end+int(tok[0])-1
            else:
                astart=start
                aend=end
                seqname=self.seqname
            if len(k_max) > 100:
                k_max=k_max[:48]+"..."+k_max[-51:]
            strout="%d\t(%s)%d\t%s\t%d\t%d"%\
                               (self.count,k_max,(abs(start-end)+1)/period,\
                                seqname,astart,aend)
            self.fileout.write("%s\n"%(strout))
            print strout
            
        elif name=="start":
            self.inStartContent=0
        elif name=="end":
            self.inEndContent=0
        elif name=="period":
            self.inPeriodContent=0
        elif name=="score":
            self.inScoreContent=0
        elif name=="unit":
            self.inUnitContent=0
            if self.type.has_key(self.unit):
                self.type[self.unit]+=1
            else:
                self.type[self.unit]=1

## class MyErrors(ErrorHandler):
    
##     def fatalError(self,exception):
##         if exception.getMessage()=="mismatched tag":
##             pass
##         else:
##             raise exception

#------------------------------------------------------------------------------

def help():

        print "\nusage:",sys.argv[0],"[ options ] <fasta filename> ..."
        print "options:"
        print "     -h: this help"
        print "     -o: fileout (default: SSRmreps.set)"
        print "     -f: error filter\n"

#------------------------------------------------------------------------------

def main ():

    outFileName = "SSRmreps.set"
    errFilter = 1.0

    opts, args = getopt.getopt(sys.argv[1:],"ho:f:")
    for o,a in opts:
        if o == "-h":
            help()
        if o == "-o":
            outFileName = a
        if o == "-f":
            errFilter = float(a)

    parser = make_parser()
    parser.setFeature( feature_namespaces, 0 )
    count = 0

    for filename in args:
        if filename == "":
            help()
            sys.exit(1)

        os.system("cutterDB -l 200000 -o 0 -w 11 "+filename)
        os.system("mreps -res 3 -exp 3.0 -maxsize 50 -xmloutput "+filename+\
                  ".mreps.xml -fasta "+filename+"_cut")

        if outFileName == "":
            outFileName = filename + ".mreps.set"
        parser.setContentHandler( FindRep( outFileName, errFilter, count ) )
        #parser.setErrorHandler(MyErrors())
        fichier = open( filename + ".mreps.xml" )
        parser.parse( fichier )
        fichier.close()
        outFile = open( outFileName )
        count = len( outFile.readlines() )
        outFile.close()
        os.system("rm -f "+filename+".mreps.xml "+filename+".Nstretch.map "+\
                  filename+"_cut")

    return 0

#------------------------------------------------------------------------------

if __name__ == "__main__": 
    main ()
