#!/usr/bin/env python
from commons.core.seq.AlignedBioseqDB import AlignedBioseqDB

import os
import sys
import getopt
from commons.core.checker.CheckerUtils import CheckerUtils
from commons.core.seq.BioseqDB import BioseqDB
from commons.core.utils.FileUtils import FileUtils


class DawgLauncher( object ):
    
    def __init__(self):
        self._seqFile = ""
        self._configFile = ""
        self._outPrefix = ""
        self._keepAlignedFasta = False
        self._verbose = 0
        
        
    def help(self):
        print
        print "usage: DawgLauncher.py [options]"
        print "options:"
        print "     -h: this help"
        print "     -s: input sequence file (format=fasta)"
        print "     -C: configuration file for DAWG"
        print "     -o: prefix of the output file (default=inFile_dawg)"
        print "     -k: keep aligned fasta as output"
        print "     -v: verbosity level (default=0/1/2)"
        print
        
        
    def setAttributesFromCmdLine(self):
        try:
            opts, args = getopt.getopt( sys.argv[1:], "hs:C:o:kv:" )
        except getopt.GetoptError, err:
            msg = str(err)
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        for o,a in opts:
            if o == "-h":
                self.help(); sys.exit(0)
            elif o == "-s":
                self._seqFile = a
            elif o == "-C":
                self._configFile = a
            elif o == "-o":
                self._outPrefix = a
            elif o =="-k":
                self._keepAlignedFasta = True
            elif o == "-v":
                self._verbose = int(a)
                
                
    def checkAttributes( self ):
        if not CheckerUtils.isExecutableInUserPath( "dawg" ):
            msg = "ERROR: 'dawg' is not in your PATH"
            sys.stderr.write( "%s\n" % msg )
            sys.exit(1)
        if self._seqFile != "" and not FileUtils.isRessourceExists(self._seqFile):
            msg = "ERROR: file '%s' do not exist" % self._seqFile
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._configFile == "":
            msg = "ERROR: missing configuration file (-C)"
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if not os.path.exists( self._configFile ):
            msg = "ERROR: config file '%s' doesn't exist" % ( self._configFile )
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._outPrefix == "":
            self._outPrefix = "%s_dawg" % ( self._seqFile )
            
            
    def addOutfileToConfig(self):
        configFileHandler = open(self._configFile, "a")
        configFileHandler.write( "File = \"%s.afa\"\n" % ( self._outPrefix ) )
        configFileHandler.close()
        
        
    def addSeqToConfig(self):
        iBsdb = BioseqDB(self._seqFile)
        if self._verbose != 0:
            print "TEsequence is %s " % iBsdb.db[0].header
        configFileHandler = open(self._configFile, "a")
        configFileHandler.write( "Sequence = \"%s\"\n" % ( iBsdb.db[0].sequence ) )
        configFileHandler.close()
        
        
    def run(self):
        self.checkAttributes()
        
        self.addOutfileToConfig()
        if self._seqFile != "":
            if self._verbose != 0:
                print " TEsequence file is %s " % self._seqFile
            self.addSeqToConfig()
            
        cmd = "dawg"
        cmd += " %s" % ( self._configFile )
        returnValue = os.system( cmd )
        if returnValue != 0:
            msg = "ERROR: 'dawg' returned '%i'" % ( returnValue )
            sys.stderr.write( "%s\n" % msg )
            sys.exit(1)
            
        absDB = AlignedBioseqDB( "%s.afa" % self._outPrefix )
        absDB.removeGaps()
        absDB.save( "%s.fa" % self._outPrefix )


        if self._verbose != 0 and self._keepAlignedFasta:
            print "daw config file is: %s" % self._configFile
            print "dawg aligned sequence is: %s.afa" % self._outPrefix 
        
        if not self._keepAlignedFasta:
            os.remove( "%s.afa" % self._outPrefix )
            
            
if __name__ == "__main__":
    i = DawgLauncher()
    i.setAttributesFromCmdLine()
    i.run()
