#!/usr/bin/env python

import os
import sys
import getopt
import ConfigParser
from decimal import *
from denovo_pipe.GenomeBackbone import GenomeBackbone
from denovo_pipe.GenomeGrammar import GenomeGrammar
from denovo_pipe.GenomeGrammar import TransitionProba


class GenomeBackboneWriter( object ):
    
    def __init__(self):
        self._mode = 1
        self._seed = 1859
        self._configFile = ""
        self._minBackboneLength = 1000
        self._maxBackboneLength = 10000
        self._maxPcentTEs = 10
        self._maxNbGenes = 1000
        self._nbSamples = 1
        self._outFile = ""
        self._verbose = 0
        
        
    def help(self):
        print
        print "usage: %s.py [options]" % ( self.__class__.__name__ )
        print "options:"
        print "     -h: this help"
        print "     -m: usage mode"
        print "         1: default, give a set of transition proba (-c) and return a backbone"
        print "         2: give backbone features, nb TEs"
        print "     -s: seed for pseudo-random number generator (default=1859)"
        print "     -c: configuration file with transition probabilities (mode 1)"
        print "     -o: name of the output file (default=GenomeBackboneWriter_<mode>.txt)"
        print "     -v: verbosity level (default=0/1/2)"
        print
        
    def setAttributesFromCmdLine(self):
        try:
            opts, args = getopt.getopt( sys.argv[1:], "hm:s:c:t:k:o:v:" )
        except getopt.GetoptError, err:
            msg = str(err)
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        for o,a in opts:
            if o == "-h":
                self.help(); sys.exit(0)
            elif o == "-m":
                self._mode = int(a)
            elif o == "-s":
                self._seed = int(a)
            elif o == "-c":
                self._configFile = a
            elif o == "-o":
                self._outFile = a
            elif o == "-v":
                self._verbose = int(a)
    
    def setConfigFile(self, configFile):
        self._configFile = configFile            
                
    def checkAttributes(self):
        if self._mode == 1:
            if self._configFile == "":
                msg = "ERROR: missing configuration file (-c)"
                sys.stderr.write( "%s\n" % msg )
                self.help()
                sys.exit(1)
            if not os.path.exists( self._configFile ):
                msg = "ERROR: can't find configuration file '%s'" % ( self._configFile )
                sys.stderr.write( "%s\n" % msg )
                self.help()
                sys.exit(1)
        if self._outFile == "":
            self._outFile = "GenomeBackboneWriter_m%i.txt" % ( self._mode )
    
    def _generateBackbone(self, seedForBackboneGeneration):
        gg = GenomeGrammar()
        return gg.generateGenomeBackbone2(seedForBackboneGeneration, self._configFile)

    
    def start(self):
        self.checkAttributes()
        if self._verbose > 0:
            msg = "seed: %i" % ( self._seed )
            if self._mode == 1:
                msg += "\nconfig: %s" % ( self._configFile )
            
    def run(self):
        self.start()
        
        iBackbone = None
        if self._mode == 1:
            iBackbone = self._generateBackbone(self._seed)
            if self._verbose > 0:
                print iBackbone.getStats()
                if self._verbose > 1:
                    print "backbone: %s" % iBackbone
                    
        elif self._mode == 2:
            print "Mode 2 Not Implemented Yet ..."
                    
        outFileHandler = open(self._outFile, "w")
        outFileHandler.write( "#%s\n" % iBackbone.getStats() )
        if self._mode == 1:
            outFileHandler.write( "#seed=%i\n" % self._seed )
        if self._mode == 2 and self._nbSamples == 1:
            outFileHandler.write( "#%s\n" % iGrammar )
        for i in iBackbone:
            outFileHandler.write( "%s\n" % i )
        outFileHandler.close()
        
        
if __name__ == "__main__":
    i = GenomeBackboneWriter()
    i.setAttributesFromCmdLine()
    i.run()
