import os,re
from pyRepet.seq.Bioseq import *
from pyRepet.seq.aligner import *
from pyRepet.coord.Path import *
from pyRepet.coord.Set import *
from pyRepet.coord.Match import *
from pyRepet.sql.RepetDBMySQL import *
from pyRepet.sql.TableAdaptator import *


#-----------------------------------------------------------------------
def align_sequence(repetdb,match_table,query_table,subject_table,\
                   qcontig_map,scontig_map,\
                   match_pen=10,mism=8,gapo=16,gape=4,gapl=20):
    align_table=match_table+"_align"
    repetdb.remove_if_exist(align_table)

    sql_cmd="CREATE TABLE %s ( path int unsigned, query_aligned_seq longtext, subject_aligned_seq longtext, score int unsigned, identity float unsigned)"\
             % (align_table)
    repetdb.execute(sql_cmd)

    sql_cmd="SELECT query_name,query_start,query_end,subject_name,subject_start,subject_end,path FROM %s ORDER BY query_name,query_start;"%\
             (match_table)
    repetdb.execute(sql_cmd)
    res=repetdb.fetchall()
    match = Match()
    nbpath=len(res)
    count=0
    for i in res:
        count+=1
        query_name=i[0]
        query_start=i[1]
        query_end=i[2]
        subject_name=i[3]
        subject_start=i[4]
        subject_end=i[5]
        match_id=i[6]
        if subject_start>subject_end:
            tmp=subject_start
            subject_start=subject_end
            subject_end=tmp
            tmp=query_start
            query_start=query_end
            query_end=tmp

        print count,"/",nbpath,"=>path",match_id,"..."
        qseq=repetdb.get_sequence_from_chr(query_table,\
                               query_name.split()[0],\
                               query_start,\
                               query_end,qcontig_map)
        sseq=repetdb.get_sequence_from_chr(subject_table,\
                               subject_name.split()[0],\
                               subject_start,\
                               subject_end,scontig_map)
        alignDB=align(qseq,sseq,\
                                          match_pen,mism,gapo,gape,gapl)

        print "header:",alignDB.db[0].header
        sc_line=re.split("Score=",alignDB.db[0].header)[1]
        score=int(sc_line.split()[0])
        
        identity=re.split("Identity=",sc_line)[1]
        if identity=="nan":
            identity="0.0"

        identity=float(identity)
        sql_cmd='INSERT INTO %s VALUES (%d,"%s","%s", %d,%f);'\
                 %(align_table,match_id,\
                   alignDB.db[0].sequence,alignDB.db[1].sequence,score,identity)
        repetdb.execute(sql_cmd)
        print "path",match_id,"Score=",score,"Identity=",identity,"ok"
            
#-----------------------------------------------------------------------
def align_path_sequence(repetdb,path_table,query_table,subject_table,\
                   qcontig_map,scontig_map,\
                   match_pen=10,mism=8,gapo=16,gape=4,gapl=20):
    align_table=path_table+"_align"
    repetdb.remove_if_exist(align_table)

    sql_cmd="CREATE TABLE %s ( path int unsigned, query_aligned_seq longtext, subject_aligned_seq longtext, score int unsigned, identity float unsigned)"\
             % (align_table)
    repetdb.execute(sql_cmd)

    qtablePathAdaptator=TablePathAdaptator(repetdb,path_table)
    path_num_list=qtablePathAdaptator.getPath_num()

    nbpath=len(path_num_list)
    count=0
    for path_num in path_num_list:
        plist=qtablePathAdaptator.getPathList_from_num(path_num)
        qlist=path_list_rangeQ2Set(plist)
        qlist=set_list_merge(qlist)
        qseq=repetdb.get_set_sequence(qlist,query_table,qcontig_map)
        slist=path_list_rangeS2Set(plist)
        sseq=repetdb.get_set_sequence(slist,subject_table,scontig_map)

        print count,"/",nbpath,"=>path",path_num,"..."

        alignDB=align(qseq,sseq,\
                                          match_pen,mism,gapo,gape,gapl)

        print "header:",alignDB.db[0].header
        sc_line=re.split("Score=",alignDB.db[0].header)[1]
        score=int(sc_line.split()[0])
        
        identity=re.split("Identity=",sc_line)[1]
        if identity=="nan":
            identity="0.0"

        identity=float(identity)
        sql_cmd='INSERT INTO %s VALUES (%d,"%s","%s", %d,%f);'\
                 %(align_table,path_num,\
                   alignDB.db[0].sequence,alignDB.db[1].sequence,score,identity)
        repetdb.execute(sql_cmd)
        print "path",path_num,"Score=",score,"Identity=",identity,"ok"
        print alignDB.db[0].sequence[:80]
        print alignDB.db[1].sequence[:80]

#-----------------------------------------------------------------------
def align_set_sequence(repetdb,set_table,query_table,subject_table,\
                   qcontig_map,\
                   match_pen=10,mism=8,gapo=16,gape=4,gapl=20):
    align_table=set_table+"_align"
    repetdb.remove_if_exist(align_table)

    sql_cmd="CREATE TABLE %s ( path int unsigned, query_aligned_seq longtext, subject_aligned_seq longtext, score int unsigned, identity float unsigned)"\
             % (align_table)
    repetdb.execute(sql_cmd)

    qtableSetAdaptator=TableSetAdaptator(repetdb,set_table)
    path_num_list=qtableSetAdaptator.getSet_num()

    nbpath=len(path_num_list)
    count=0
    for path_num in path_num_list:
        qlist=qtableSetAdaptator.getSetList_from_num(path_num)
        #pyRepet.coord.Set.set_list_show(qlist) # debug
        qlist=set_list_merge(qlist)
        set_list_show(qlist) # debug
        qseq=repetdb.get_set_sequence(qlist,query_table,qcontig_map)

        sseq=repetdb.get_sequence(subject_table,qlist[0].name)
        
        print count,"/",nbpath,"=>path",path_num,"..."

        alignDB=align(qseq,sseq,\
                                          match_pen,mism,gapo,gape,gapl)

        print "header:",alignDB.db[0].header
        sc_line=re.split("Score=",alignDB.db[0].header)[1]
        score=int(sc_line.split()[0])
        
        identity=re.split("Identity=",sc_line)[1]
        if identity=="nan":
            identity="0.0"

        identity=float(identity)
        sql_cmd='INSERT INTO %s VALUES (%d,"%s","%s", %d,%f);'\
                 %(align_table,path_num,\
                   alignDB.db[0].sequence,alignDB.db[1].sequence,score,identity)
        repetdb.execute(sql_cmd)
        print "path",path_num,"Score=",score,"Identity=",identity,"ok"
        print alignDB.db[0].sequence[:80]
        print alignDB.db[1].sequence[:80]
